package com.jiuqi.syntax;

import java.io.PrintStream;
import java.util.HashMap;

public class NiporlanItemFactory {
   public PrintStream logger = null;
   private long id = System.currentTimeMillis();
   private static NiporlanItemFactory globalFactory = new NiporlanItemFactory();
   private static boolean useGlobalFactory = true;
   protected static HashMap OP_NIS = new HashMap();
   protected HashMap FUNC_NIS = new HashMap();
   protected static NiporlanItem BOOL_TRUE_NI;
   protected static NiporlanItem BOOL_FALSE_NI;
   protected static NiporlanItem BOOL_NULL_NI;
   public int FOUND_COUNT = 0;
   public int MISSED_COUNT = 0;
   public int NULL_COUNT = 0;
   protected HashMap[] dataCellCache = null;
   protected static int PAGE_COUNT;

   static {
      for(int i = 1; i <= 20; ++i) {
         OP_NIS.put(new Integer(i), new NiporlanItem(i));
      }

      CommonData cd = new CommonData();
      cd.type = 5;
      cd.isNull = false;
      cd.cdBool = true;
      BOOL_TRUE_NI = new NiporlanItem(cd);
      cd = new CommonData();
      cd.type = 5;
      cd.isNull = false;
      cd.cdBool = false;
      BOOL_FALSE_NI = new NiporlanItem(cd);
      cd = new CommonData();
      cd.type = 5;
      cd.isNull = true;
      BOOL_NULL_NI = new NiporlanItem(cd);
      PAGE_COUNT = 2000;
   }

   public static boolean getUseGlobalFactory() {
      return useGlobalFactory;
   }

   public static void setUseGlobalFactory(boolean value) {
      useGlobalFactory = value;
   }

   public static NiporlanItemFactory getFactory() {
      return useGlobalFactory ? globalFactory : new NiporlanItemFactory();
   }

   public NiporlanItemFactory() {
      if (this.logger != null) {
         this.logger.println(Thread.currentThread().getName() + "\t" + "Crate NO." + this.id + " NIFactory.");
      }

   }

   public NiporlanItem getNiporlanItem(int opCode) {
      NiporlanItem ni = null;
      synchronized(OP_NIS) {
         ni = (NiporlanItem)OP_NIS.get(new Integer(opCode));
         if (ni == null) {
            ni = new NiporlanItem(opCode);
            OP_NIS.put(new Integer(opCode), ni);
            if (this.logger != null) {
               this.logger.println(Thread.currentThread().getName() + "\t" + "添加新的操作符(" + opCode + ")共享NiporlanItem ");
            }
         }

         return ni;
      }
   }

   public NiporlanItem getNiporlanItem(String funcName) {
      if (funcName == null) {
         return null;
      } else {
         NiporlanItem ni = null;
         synchronized(this.FUNC_NIS) {
            ni = (NiporlanItem)this.FUNC_NIS.get(funcName);
            if (ni == null) {
               ni = new NiporlanItem(funcName);
               this.FUNC_NIS.put(funcName, ni);
               if (this.logger != null) {
                  this.logger.println(Thread.currentThread().getName() + "\t" + "添加新的函数(" + funcName + ")共享NiporlanItem ");
               }
            }

            return ni;
         }
      }
   }

   public NiporlanItem getNiporlanItem(CommonData commonData) {
      if (commonData != null && commonData.type == 5) {
         if (commonData.isNull) {
            return BOOL_NULL_NI;
         }

         if (commonData.cdBool) {
            return BOOL_TRUE_NI;
         }

         if (!commonData.cdBool) {
            return BOOL_FALSE_NI;
         }
      }

      return new NiporlanItem(commonData);
   }

   public NiporlanItem getNiporlanItem(DataCell dataCell) {
      if (this.dataCellCache == null) {
         this.dataCellCache = new HashMap[PAGE_COUNT];

         for(int i = 0; i < this.dataCellCache.length; ++i) {
            this.dataCellCache[i] = new HashMap();
         }

         if (this.logger != null) {
            this.logger.println(Thread.currentThread().getName() + "\t" + "DataCell缓冲池初始化 " + System.currentTimeMillis());
         }
      }

      if (dataCell == null) {
         if (this.logger != null) {
            this.logger.println(" DataCell is null");
         }

         return new NiporlanItem(dataCell);
      } else {
         HashMap page = null;
         int index = dataCell.tableOffset;
         if (index < 0) {
            index = -index;
         }

         page = this.dataCellCache[index % PAGE_COUNT];
         DataCell oldDataCell = null;
         String mapKey = dataCell.toString();
         oldDataCell = (DataCell)page.get(mapKey);
         if (oldDataCell == null) {
            synchronized(page) {
               if (this.logger != null) {
                  this.logger.println(Thread.currentThread().getName() + "\t" + "未命中 " + mapKey);
               }

               oldDataCell = (DataCell)page.get(mapKey);
               if (oldDataCell == null) {
                  if (this.logger != null) {
                     this.logger.println(Thread.currentThread().getName() + "\t" + "连续未命中 " + mapKey);
                  }

                  DataCell newDataCell = new DataCell();
                  newDataCell.assign(dataCell);
                  page.put(mapKey, newDataCell);
               }
            }
         }

         return oldDataCell != null && oldDataCell.statItem == dataCell.statItem && oldDataCell.offset == dataCell.offset ? new NiporlanItem(oldDataCell) : new NiporlanItem(dataCell);
      }
   }

   public NiporlanItem getNiporlanItem(Object obj) {
      return new NiporlanItem(obj);
   }

   public String getProductCountDesc() {
      return "Factory NO." + this.id;
   }

   public void destroy() {
      if (!useGlobalFactory && this != globalFactory) {
         this.FUNC_NIS.clear();
      }

   }
}
